;------------------------------------------------------
; Sol_Asm assembler core
; Copyright (c) 2004-2005, Bogdan Valentin Ontanu.
; All rights reserved.
;------------------------------------------------------


;******************************
; Encode Jump's
;******************************

;====================================
; For Simple jumps 
; without Mod_R/M or SIB
;====================================
On_Mode_JMP PROC
	USES	esi,edi
	ARG 	@@token_start,@@token_size:dword

	ODS1	<"On Mode Jump">

	;-----------------------
	; jmp eax
	;-----------------------
	Call	Check_Register,[@@token_start],[@@token_size]
	.IF eax!=-1
		ODS1	<"Register Found">
		ODV	ebx

		Call	Encode_JMP_Reg,ebx
		mov	[parse_mode],P2_MODE_DEFAULT
		ret		
	.ENDIF

	;-----------------------
	; jmp label
	;-----------------------	
	Call	Check_Label,[@@token_start],[@@token_size]
	.IF eax!=-1
		Call	Encode_JMP_Label,ebx
	.ELSE
		.IF [pass_nr]==1
			ODS1	<"Encoding Dummy Jump">
			mov	eax,[addr_crt]
			add	eax,5
			Call	Encode_JMP_Label,eax	
		.ELSE
			;-----------------------
			; jmp number
			;-----------------------
			Call	Check_Value,[@@token_start],[@@token_size]
			.IF eax==-1
				Call	Print_Error,offset sz_err_012,[@@token_start],[@@token_size]			
			.ELSE
				Call	Encode_JMP_Label,ebx	
			.ENDIF
		.ENDIF
	.ENDIF

	mov	[parse_mode],P2_MODE_DEFAULT
	ret
ENDP

;=============================
; jmp [xxx]
; requires Mod_R/M and/or SIB
;=============================
On_Mode_JMP_Br PROC
	USES	esi,edi
	ARG 	@@token_start,@@token_size:dword


	ODS1	<"On Mode Jump []">

	mov	esi,[@@token_start]
	mov	ecx,[@@token_size]
	inc	esi
	sub	ecx,2
	.IF ZERO?
		Call	Print_Error,offset sz_err_017,[@@token_start],[@@token_size]
		mov	[parse_mode],P2_MODE_DEFAULT
		ret		
	.ENDIF

	;-------------------------------
	; a little preparation
	;-------------------------------
	Call	Trim_Expression,esi,ecx
	mov	[@@token_start],eax
	mov	[@@token_size],ebx

	;ODS1	<"Trimed Expression">
	;Call	Str_Copy_Nr,[@@token_start],offset sz_tmp1,[@@token_size]
	;ODS	<offset sz_tmp1>
	;ODV	[@@token_size]

	;-------------------------------
	;jmp [eax]
	;-------------------------------
	Call	Check_Register,[@@token_start],[@@token_size]
	.IF eax!=-1
		Call	Encode_JMP_Indirect_Reg,ebx
		mov	[parse_mode],P2_MODE_DEFAULT
		ret	
	.ENDIF

	;---------------------------
	;jmp [label]
	;---------------------------
	Call	Check_Label,[@@token_start],[@@token_size]
	.IF eax!=-1
		Call	Encode_JMP_Indirect_Mem,ebx
		mov	[parse_mode],P2_MODE_DEFAULT
		ret			
	.ENDIF

	;------------------------------------
	;jmp [disp32]
	;------------------------------------
	Call	Check_Value,[@@token_start],[@@token_size]
	.IF eax!=-1
		Call	Encode_JMP_Indirect_Mem,ebx
		mov	[parse_mode],P2_MODE_DEFAULT
		ret			
	.ENDIF

	;------------------------
	; it is NOT simple :(
	; requires ModRM
	;------------------------
	Call	Parse_ModRM,[@@token_start],[@@token_size]
	.IF eax==-1
		Call	Print_Error,offset sz_err_021,[@@token_start],[@@token_size]
		mov	[parse_mode],P2_MODE_DEFAULT
		ret		
	.ELSE
		Call	Encode_JMP_ModRM
		mov	[parse_mode],P2_MODE_DEFAULT
		ret
	.ENDIF

@@finish:
	Call	Print_Error,offset sz_err_018,[@@token_start],[@@token_size]
	mov	[parse_mode],P2_MODE_DEFAULT
	ret					

ENDP


;==============================
;JMP	label_01
;==============================
Encode_JMP_Label PROC
	USES	ebx,esi,edi
	ARG	@@jmp_here:dword

	mov	edi,[out_crt]
	add	edi,[lp_out_buff]

	;--------------------------
	; write opcode
	;--------------------------
	mov	al,0E9h
	mov	byte ptr [edi],al

	inc	edi
	inc	[out_crt]	
	inc	[addr_crt]

	;-----------------------
	; calc displacement
	;-----------------------
	mov	ebx,[addr_crt]
	add	ebx,4	
	mov	eax,[@@jmp_here]
	sub	eax,ebx

	ODS1	<"Jump Displacement:">
	ODV	eax

	mov	dword ptr [edi],eax
	
	add	[addr_crt],4
	add	[out_crt],4	

	ret
ENDP

;================================
;JMP	eax
;================================
Encode_JMP_Reg PROC
	USES	ebx,esi,edi
	ARG	@@register:dword


	mov	edi,[out_crt]
	add	edi,[lp_out_buff]

	;--------------------------
	; write opcode
	;--------------------------
	mov	al,0FFh
	mov	byte ptr [edi],al

	inc	edi
	inc	[out_crt]	
	inc	[addr_crt]

	;-----------------------
	; calc register
	;-----------------------
	mov	eax,[@@register]
	or	eax,11100000b		;Mod=11, ext=100,M/R=reg
	mov	byte ptr [edi],al

	add	[addr_crt],1
	add	[out_crt],1	

	ret
ENDP

;================================
;JMP	[eax]
;================================
Encode_JMP_Indirect_Reg PROC
	USES	ebx,esi,edi
	ARG	@@register:dword


	mov	edi,[out_crt]
	add	edi,[lp_out_buff]

	;--------------------------
	; write opcode
	;--------------------------
	mov	al,0FFh
	mov	byte ptr [edi],al

	inc	edi
	inc	[out_crt]	
	inc	[addr_crt]

	;-----------------------
	; calc register
	; Mod=00, extop=100 R/M=reg
	;-----------------------
	mov	eax,[@@register]
	or	eax,00100000b		
	mov	byte ptr [edi],al

	add	[addr_crt],1
	add	[out_crt],1	

	ret
ENDP

;==============================
;JMP [mem_location]
;==============================
Encode_JMP_Indirect_Mem PROC
	USES	ebx,esi,edi
	ARG	@@jmp_by_here:dword

	ODS1	<"Encode JMP Indirect Mem">

	mov	edi,[out_crt]
	add	edi,[lp_out_buff]
	;--------------------------
	; write opcode
	;--------------------------
	mov	al,0FFh
	mov	byte ptr [edi],al

	inc	edi
	inc	[out_crt]	
	inc	[addr_crt]

	;-----------------------
	; Write Mod R/M
	;-----------------------
	mov	eax,00100101b		;Mod=00, ext=100 R/M=101=5=disp32
	mov	byte ptr [edi],al

	inc	edi
	add	[addr_crt],1
	add	[out_crt],1	

	;----------------------------
	; displacement32
	;----------------------------
	mov	eax,[@@jmp_by_here]
	mov	dword ptr [edi],eax
	
	add	[addr_crt],4
	add	[out_crt],4	

	ret
ENDP

Encode_JMP_ModRM PROC
	USES 	esi,edi

	ODS1	<"+++Encode JMP ModRM">

	ODS1	<"Modrm scale:">
	ODV	[modrm_scale]
	ODV	[flag_modrm_scale]

	ODS1	<"Modrm index:">
	ODV	[modrm_index]	
	ODV	[flag_modrm_index]
		
	ODS1	<"Modrm base:">
	ODV	[modrm_base]
	ODV	[flag_modrm_base]

	ODS1	<"Modrm Displ:">
	ODV	[modrm_disp]
	ODV	[flag_modrm_disp]

	.IF [factor_nr]==2

		ODS1	<"Factors 2">

		mov	eax,[flag_modrm_base]
		and	eax,[flag_modrm_disp]
		.IF !ZERO?
			ODS1	<"Base+Disp">
			;-------------------
			; simple base+disp
			;-------------------
			mov	edi,[out_crt]
			add	edi,[lp_out_buff]
			;--------------------------
			; write opcode
			;--------------------------
			mov	al,0FFh
			mov	byte ptr [edi],al

			inc	edi
			inc	[out_crt]	
			inc	[addr_crt]

			;-----------------------
			; Write Mod R/M
			;-----------------------
			mov	eax,10100000b		;Mod=10, ext=100 R/M=reg
			or	eax,[modrm_base]
			mov	byte ptr [edi],al

			inc	edi
			add	[addr_crt],1
			add	[out_crt],1	

			;----------------------------
			; displacement32
			;----------------------------
			mov	eax,[modrm_disp]
			mov	dword ptr [edi],eax
	
			add	[addr_crt],4
			add	[out_crt],4			
			ret

		.ENDIF

		mov	eax,[flag_modrm_base]
		and	eax,[flag_modrm_index]
		.IF !ZERO?
			ODS1	<"Base+Index">
			;-------------------
			;simple base+index
			;no displacement
			;-------------------
			mov	edi,[out_crt]
			add	edi,[lp_out_buff]
			;--------------------------
			; write opcode
			;--------------------------
			mov	al,0FFh
			mov	byte ptr [edi],al

			inc	edi
			inc	[out_crt]	
			inc	[addr_crt]

			;---------------------------------------
			;Write Mod R/M
			;Mod=00, ext=100 R/M=100=SIB follows
			;---------------------------------------
			mov	eax,00100100b		
			mov	byte ptr [edi],al

			inc	edi
			add	[addr_crt],1
			add	[out_crt],1	

			;----------------------------
			; Scale
			;----------------------------
			mov	eax,[modrm_scale]
			.IF eax==0
				mov	ebx,0
			.ELSEIF eax==2
				mov	ebx,01000000b
			.ELSEIF eax==4
				mov	ebx,10000000b
			.ELSEIF eax==8
				mov	ebx,11000000b
			.ELSE
				;---------------
				;error
				;---------------
				mov	ebx,0
			.ENDIF

			;---------------------------
			; write SIB
			;---------------------------
			mov	eax,[modrm_index]
			shl	eax,3
			or	eax,[modrm_base]
			or	eax,ebx	

			mov	byte ptr [edi],al
			
			inc	edi
			add	[addr_crt],1
			add	[out_crt],1		

			ret
		.ENDIF


		mov	eax,[flag_modrm_index]
		and	eax,[flag_modrm_disp]
		.IF !ZERO?
			ODS1	<"Index+Displ">
			;-------------------
			;index
			;+displacement
			;-------------------
			mov	edi,[out_crt]
			add	edi,[lp_out_buff]
			;--------------------------
			; write opcode
			;--------------------------
			mov	al,0FFh
			mov	byte ptr [edi],al

			inc	edi
			inc	[out_crt]	
			inc	[addr_crt]

			;---------------------------------------
			;Write Mod R/M
			;Mod=00, ext=100 R/M=100=SIB follows
			;---------------------------------------
			mov	eax,00100100b		
			mov	byte ptr [edi],al

			inc	edi
			add	[addr_crt],1
			add	[out_crt],1	

			;----------------------------
			; Scale
			;----------------------------
			mov	eax,[modrm_scale]
			.IF eax==0
				mov	ebx,0
			.ELSEIF eax==2
				mov	ebx,01000000b
			.ELSEIF eax==4
				mov	ebx,10000000b
			.ELSEIF eax==8
				mov	ebx,11000000b
			.ELSE
				;---------------
				;error
				;---------------
				mov	ebx,0
			.ENDIF

			;---------------------------
			; write SIB
			;---------------------------
			mov	eax,[modrm_index]
			shl	eax,3
			or	eax,101b		;base=none if mod=00
			or	eax,ebx	

			mov	byte ptr [edi],al
			
			inc	edi
			add	[addr_crt],1
			add	[out_crt],1		


			;----------------------------
			; displacement32
			;----------------------------
			mov	eax,[modrm_disp]
			mov	dword ptr [edi],eax
	
			add	[addr_crt],4
			add	[out_crt],4	

			ret
		.ENDIF

	.ELSEIF [factor_nr]==3
			;----------------------------
			;most complicated version
			;----------------------------
			mov	edi,[out_crt]
			add	edi,[lp_out_buff]
			;--------------------------
			; write opcode
			;--------------------------
			mov	al,0FFh
			mov	byte ptr [edi],al

			inc	edi
			inc	[out_crt]	
			inc	[addr_crt]

			;-----------------------
			; Write Mod R/M
			;-----------------------
			mov	eax,10100100b		;Mod=10, ext=100 R/M=100=4=disp32+SIB
			mov	byte ptr [edi],al

			inc	edi
			add	[addr_crt],1
			add	[out_crt],1	

	
			mov	eax,[modrm_scale]

			.IF eax==0
				mov	ebx,0
			.ELSEIF eax==2
				mov	ebx,01000000b
			.ELSEIF eax==4
				mov	ebx,10000000b
			.ELSEIF eax==8
				mov	ebx,11000000b
			.ELSE
				;---------------
				;error
				;---------------
				mov	ebx,0
			.ENDIF

			mov	eax,[modrm_index]
			shl	eax,3
			or	eax,ebx
			or	eax,[modrm_base]

			ODS1	<"SIB:">
			ODVh	eax

			mov	byte ptr [edi],al
			
			inc	edi
			add	[addr_crt],1
			add	[out_crt],1
			;----------------------------
			; displacement32
			;----------------------------
			mov	eax,[modrm_disp]
			mov	dword ptr [edi],eax
	
			add	[addr_crt],4
			add	[out_crt],4			
	.ENDIF


	ret
ENDP
