;------------------------------------------------------
; Sol_Asm assembler core
; Copyright (c) 2004-2005, Bogdan Valentin Ontanu.
; All rights reserved.
;------------------------------------------------------


;=====================================
; tokens used for parsing 
; instructions and directives
;=====================================
TYPE_OPCODE		EQU	1b
TYPE_DIRECTIVE		EQU	10b
TYPE_INTER		EQU	100b

.data
	
token_list:

	db	4,"byte"
	dd	$+12
	dd	TYPE_DIRECTIVE+TYPE_INTER
	dd	offset On_Token_BYTE

	db	4,"word"
	dd	$+12
	dd	TYPE_DIRECTIVE+TYPE_INTER
	dd	offset On_Token_WORD

	db	5,"dword"
	dd	$+12
	dd	TYPE_DIRECTIVE+TYPE_INTER
	dd	offset On_Token_DWORD

	db	3,"ptr"
	dd	$+12
	dd	TYPE_DIRECTIVE+TYPE_INTER
	dd	offset On_Token_PTR

	db	3,"mov"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset	On_Token_MOV

	db	5,"struc"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset	On_Token_STRU

	db	4,"ends"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset	On_Token_ENDS

	db	4,"proc"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset	On_Token_PROC

	db	4,"uses"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset On_Token_USES

	db	3,"arg"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset On_Token_ARG

	db	5,"local"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset On_Token_LOCAL

	db	4,"endp"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset	On_Token_ENDP			

	db	4,"call"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset On_Token_CALL

	db	3,"jmp"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset On_Token_JMP


	db	3,"org"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset	On_Token_ORG

	db	4,"disp"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset	On_Token_DISP


	db	3,"cli"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0fah

	db	3,"rep"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0F3h

	db	3,"ret"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Token_RET
	dd	0C3h

	db	4,"retn"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset On_Token_RETN
	
	db	7,"include"
	dd	$+12
	dd	TYPE_DIRECTIVE
	dd	offset On_Include_File

	db	2,"db"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset	On_Define_Byte

	db	2,"dw"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset	On_Define_Word

	db	2,"dd"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset	On_Define_Dword
	
	db	3,"nop"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	90h

	db	3,"cbw"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	098h

	db	3,"cdq"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	099h

	db	3,"cld"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0FCh

	db	3,"clc"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0F8h

	db	3,"cmc"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0F5h

	db	3,"hlt"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0F4h

	db	4,"int3"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0CCh

	db	4,"into"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0CEh

	db	4,"iret"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0CFh

	db	5,"iretd"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0CFh

	db	3,"aaa"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	037h

	db	5,"cpuid"
	dd	$+20
	dd	TYPE_OPCODE
	dd	offset On_Inst_2byte
	dd	00Fh,0A2h

	db	5,"rdtsc"
	dd	$+20
	dd	TYPE_OPCODE
	dd	offset On_Inst_2byte
	dd	00Fh,031h

	db	3,"aad"
	dd	$+20
	dd	TYPE_OPCODE
	dd	offset On_Inst_2byte
	dd	0D4h,0Ah

	db	3,"aam"
	dd	$+20
	dd	TYPE_OPCODE
	dd	offset On_Inst_2byte
	dd	0D5h,0Ah

	db	3,"aas"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	03Fh

	db	4,"lahf"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	09Fh

	db	5,"leave"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0C9h

	db	6,"pushad"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	060h

	db	5,"pusha"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	060h

	db	6,"pushfd"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	09Ch

	db	5,"pushf"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	09Ch

	db	3,"pop"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset On_Token_POP

	db	5,"popad"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	061h

	db	4,"popa"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	061h

	db	5,"popfd"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	09Dh

	db	4,"popf"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	09Dh

	db	4,"push"
	dd	$+12
	dd	TYPE_OPCODE
	dd	offset On_Token_PUSH

	db	4,"lock"
	dd	$+16
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0f0h


	;------------------------------------
	; Last token MUST have next link==0
	;------------------------------------
	db	3,"sti"
	dd	0		;next link
	dd	TYPE_OPCODE
	dd	offset On_Inst_1byte
	dd	0fbh

.code


On_Token_BYTE PROC
	ARG	@@param:dword
	ODS1	<"+++On Token BYTE">
	mov	[flag_byte],1
	mov	eax,1
	ret
ENDP

On_Token_WORD PROC
	ARG	@@param:dword
	ODS1	<"+++On Token WORD">
	mov	[flag_word],1
	mov	eax,2
	ret
ENDP

On_Token_DWORD PROC
	ARG	@@param:dword
	ODS1	<"+++On Token DWORD">
	mov	[flag_dword],1
	mov	eax,4
	ret
ENDP

On_Token_PTR PROC
	ARG	@@param:dword
	ODS1	<"+++On Token PTR">
	mov	[flag_ptr],1

	mov	eax,0
	ret
ENDP

On_Token_MOV PROC
	ARG	@@param:dword
	ODS1	<"+++On Token MOV">
	mov	[parse_mode],P2_MODE_MOV1
	;----------------------
	;clear flags
	;----------------------
	xor	eax,eax
	mov	[flag_ptr],eax
	mov	[flag_byte],eax
	mov	[flag_word],eax	
	mov	[flag_dword],eax
	ret
ENDP

On_Token_ARG PROC
	ARG	@@param:dword
	ODS1	<"+++On Token ARG">
	mov	[parse_mode],P2_MODE_ARG
	ret
ENDP

On_Token_CALL PROC
	ARG	@@param:dword
	ODS1	<"On CALL">
	mov	[parse_mode],P2_MODE_CALL
	ret
ENDP

On_Token_DISP PROC
	ARG	@@param:dword
	ODS1	<"+++On Token DISP">
	mov	[parse_mode],P2_MODE_DISP
	ret
ENDP

On_Token_ENDP PROC
	ARG	@@param:dword
	ODS1	<"+++On Token ENDP">

	.IF [proc_crt]==-1
		Call	Print_Error1,offset sz_err_032
	.ENDIF
	mov	[parse_hll_mode],P2_HLL_DEFAULT
	mov	[proc_crt],-1
	ret
ENDP

On_Token_JMP PROC
	ARG	@@param:dword
	ODS1	<"On Token JMP">
	mov	[parse_mode],P2_MODE_JMP
	ret
ENDP

On_Token_LOCAL PROC
	ARG	@@param:dword
	ODS1	<"+++On Token LOCAL">
	mov	[parse_mode],P2_MODE_LOCAL
	ret
ENDP

On_Token_ORG PROC
	ARG	@@param:dword
	ODS1	<"+++On Token ORG">
	mov	[parse_mode],P2_MODE_ORG
	ret
ENDP

On_Token_PUSH PROC
	ARG	@@param:dword
	ODS1	<"+++On Token PUSH">
	mov	[parse_mode],P2_MODE_PUSH
	ret
ENDP

On_Token_POP PROC
	ARG	@@param:dword
	ODS1	<"+++On Token POP">
	mov	[parse_mode],P2_MODE_POP
	ret
ENDP

On_Token_STRU PROC
	ARG	@@param:dword
	ODS1	<"+++On Token STRU">
	mov	[parse_mode],P2_MODE_STRU
	mov	[parse_hll_mode],P2_HLL_STRU_START
	ODV	[parse_mode]
	ODV	[parse_hll_mode]
	ret
ENDP

On_Token_ENDS PROC
	ARG	@@param:dword

	ODS1	<"+++On Token ENDS">

	.IF [stru_crt]==-1
		Call	Print_Error1,offset sz_err_034
	.ENDIF

	mov	[parse_hll_mode],P2_HLL_DEFAULT
	mov	[parse_mode],P2_MODE_DEFAULT
	mov	[stru_crt],-1

	ret
ENDP

On_Token_PROC PROC
	ARG	@@param:dword
	ODS1	<"+++On Token PROC">
	mov	[parse_hll_mode],P2_HLL_PROC_START
	ret
ENDP

On_Token_RET PROC
	ARG		@@param:dword

	ODS1	<"+++On Token RET">

	.IF [parse_hll_mode]==P2_HLL_PROC_START
		Call	Generate_Epilogue
		ret
		
	.ELSEIF [parse_hll_mode]==P2_HLL_PROC_BODY
		Call	Generate_Epilogue
		ret
		
	.ELSE
		;-----------------------------------
		;normal ret, aka not inside a PROC
		;just plain encode
		;-----------------------------------
		Call	Encode_RET
	.ENDIF

	ret
ENDP

On_Token_RETN PROC
	ARG	@@param:dword
	ODS1	<"+++On Token RETN">
	mov	[parse_mode],P2_MODE_RETN
	ret
ENDP

On_Token_USES PROC
	ARG	@@param:dword
	ODS1	<"+++On Token USES">
	mov	[parse_mode],P2_MODE_USES
	ret
ENDP

On_Define_Byte PROC
	ARG	@@param:dword

	ODS1	"+++On Define Byte"
	mov	[parse_mode],P2_MODE_DEFINE_B
	
	ret
ENDP

On_Define_Word PROC
	ARG	@@param:dword

	ODS1	"+++On Define Word"
	mov	[parse_mode],P2_MODE_DEFINE_W
	
	ret
ENDP

On_Define_Dword PROC
	ARG	@@param:dword

	ODS1	"+++On Define Dword"
	mov	[parse_mode],P2_MODE_DEFINE_D		
	
	ret
ENDP

On_Include_File PROC
	ARG	@@param:dword

	ODS1	"+++On include file"
	mov	[parse_mode],P2_MODE_INCLUDE

	ret
ENDP



